#!/usr/bin/python
#
# Travesty, a hardware address management tool for network devices
#
# http://www.cse.msstate.edu/~rwm8/travesty/
#
# Copyright 2005 Robert Wesley McGrew
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import curses
import time
import commands
import sys
import random
import os

version = '1.0'

#############################
# Edit these to point to 
# where you have your scripts
# and files
vendorfile = 'manuf'
addrdb = 'addrdb'
scriptdir = 'addons'

#############################
# Menus
#
mainmenu =  ['Change Device',
				 'Manual Entry',
				 'Generate Random',
				 'Change Vendor Prefix',
				 'Load from Address Database',
				 'Addon Scripts',
				 'Set Address',
				 'Set Address and Exit',
				 'Exit Without Changes']

####################################################
#Functions

##########################
#Loads the vendor file
#into a list suitable for
#menus
#
def loadVendorPrefix():
	vendorlist = []
	f=open(vendorfile,'r')
	lines = f.readlines()
	for line in lines:
		if line[0] == '#':
			continue
		if len(line) < 7:
			continue
		prefix = ''
		i = 0
		while not line[i].isspace():
			prefix += line[i]
			i+=1
		vendor = ''
		i+=1
		while not line[i].isspace():
			vendor += line[i]
			i+=1
		while len(vendor) < 10:
			vendor += ' '
		comments = line[i:]
		comments = comments.strip()
		entry = prefix + ' | ' + vendor + ' | ' + comments
		vendorlist.append(entry[:70])
	return vendorlist


##########################
# Parses the current
# hardware address from
# ifconfig
def getCurrentAddr(device):
	output = commands.getoutput('ifconfig ' + device)
	if output.find('error fetching') != -1:
		return 'error'
	addrptr = output.find('HWaddr') + 7
	addr = output[addrptr:addrptr+17]
	return addr

##########################
# Set hardware address
def setCurrentAddr(device,addr):
	output = commands.getoutput('ifconfig ' + device + ' hw ether ' + addr)
	if output.find('error fetching') != -1:
		return 'error'
	return addr

##########################
# Generate a random
# hardware address
def generateRandomAddr():
	random.seed()
	addr = ''
	a = hex(random.randint(0,255))[2:]
	if len(a) < 2:
		a = '0' + a
	addr += a
	for i in range(0,5):
		a = hex(random.randint(0,255))[2:]
		if len(a) < 2:
			a = '0' + a
		addr += ':' + a
	addr = addr.upper()
	return addr

##########################
# Grab a line of text 
# input
def getTextInput(parentwindow, caption, width):
	textwin = parentwindow.subwin(4,width,((max_y-1)-4)/2,((max_x-1)-width)/2)
	textwin.clear()
	textwin.box()
	textwin.addstr(1,1,caption)
	textwin.noutrefresh()
	curses.doupdate()
	curses.curs_set(1)
	curses.echo()
	response = textwin.getstr(2,1,width-2)
	curses.noecho()
	curses.curs_set(0)
	del textwin
	parentwindow.clear()
	return response

	
##########################
# Presents a menu in a 
# curses window from a 
# list.
def presentMenu(window, menu, width, center):
	selection = 0
	max_y, max_x =  window.getmaxyx()
	maxmenuitems = max_y-2;
	numitems = len(menu);
	while True:
		if selection > maxmenuitems/2 and numitems > maxmenuitems:
			start = selection - maxmenuitems/2
			if(numitems-selection < maxmenuitems/2):
				start = numitems - maxmenuitems
		else:
			start = 0
		if start + maxmenuitems/2 < numitems and numitems > maxmenuitems:
			end = start + maxmenuitems
		else:
			end = start + numitems
		pos = 0
		
		eraser = ''
		for i in range(0,width):
			eraser += ' '
		for i in range(start,end):
			if center:
				window.addstr(pos+1,1+((max_x-1)-len(eraser))/2,eraser)
			else:
				window.addstr(pos+1,0,eraser)
			if i == selection:
				if center:
					window.addstr(pos+1,1+((max_x-1)-len(menu[i][:width]))/2,menu[i][:width],
							         curses.A_REVERSE)
				else:
					window.addstr(pos+1,0,menu[i][:width],curses.A_REVERSE)
			else:
				if center:
					window.addstr(pos+1,1+((max_x-1)-len(menu[i][:width]))/2,menu[i][:width])
				else:
					window.addstr(pos+1,0,menu[i][:width])
			pos = pos + 1
		
		window.noutrefresh()
		curses.doupdate()
		
		c = stdscr.getch()
		if c == curses.KEY_UP or c == ord('k'):
			selection = selection - 1
			if selection < 0:
				selection = 0
		elif c == curses.KEY_DOWN or c == ord('j'):
			selection = selection + 1
			if selection > numitems-1:
				selection = numitems-1
		elif c == 10 or c == curses.KEY_ENTER or c == ord('l'):
			break;
		elif c == curses.KEY_NPAGE:
			selection = selection + maxmenuitems-1
			if selection > numitems-1:
				selection = numitems-1
		elif c == curses.KEY_PPAGE:
			selection = selection - maxmenuitems-1
			if selection < 0:
				selection = 0
		elif c == curses.KEY_HOME:
			selection = 0
		elif c == curses.KEY_END:
			selection = numitems -1
		elif c == ord('/') or c == ord('f'):
			filteredmenu = []
			filter = getTextInput(window,'Input search string:',70)
			for item in menu:
				if filter.upper() in item.upper():
					filteredmenu.append(item)
			menu = filteredmenu
			selection = 0
			numitems = len(menu)
			window.clear()
			window.noutrefresh()
			curses.doupdate()
		elif c == ord('q'):
			return 'quit'
			break;
	return menu[selection]


########################################################################
# Main code

if(len(sys.argv) != 2):
	dev = 'eth0'
else:
	dev = sys.argv[1]

stdscr = curses.initscr()
curses.noecho()
curses.cbreak()
stdscr.keypad(1)
curses.curs_set(0)
menulocation = 0

max_y, max_x = stdscr.getmaxyx()

menuwin = stdscr.subwin(max_y-9,max_x-3,8,2)
newaddr = getCurrentAddr(dev)
while True:
	stdscr.clear()
	stdscr.box()
	title = 'Travesty v' + version
	stdscr.addstr(1,1+((max_x-1)-len(title))/2,title)
	author = 'http://www.cse.msstate.edu/~rwm8/travesty/'
	stdscr.addstr(2,1+((max_x-1)-len(author))/2,author)
	max_y, max_x = stdscr.getmaxyx()
	configline = 'Configuring: ' + dev
	stdscr.addstr(4,1+((max_x-1)-len(configline))/2,configline)
	addr = getCurrentAddr(dev)
	currentline = 'Current Address: ' + addr
	stdscr.addstr(5,1+((max_x-1)-len(currentline))/2,currentline)
	newline = '    New Address: ' + newaddr
	stdscr.addstr(6,1+((max_x-1)-len(newline))/2,newline)
	
	selection = presentMenu(menuwin,mainmenu,40,True)
	if selection == 'Exit Without Changes':
		break
	elif selection == 'Change Device':
		dev = getTextInput(stdscr,'Network Device Name (ex. eth0):',42)
		newaddr = addr = getCurrentAddr(dev)
	elif selection == 'Manual Entry':
		newaddr = getTextInput(stdscr,'Hardware Address:',42)
	elif selection == 'Generate Random':
		newaddr = generateRandomAddr()
	elif selection == 'Set Address':
		setCurrentAddr(dev,newaddr)
		newaddr = addr = getCurrentAddr(dev)
	elif selection == 'Set Address and Exit':
		setCurrentAddr(dev,newaddr)
		newaddr = addr = getCurrentAddr(dev)
		break
	elif selection == 'Change Vendor Prefix':
		menuwin.clear()
		vendormenu = loadVendorPrefix()
		vendormenuwin = stdscr.subwin(max_y-9,max_x-3,8,2)
		newvendor = presentMenu(vendormenuwin,vendormenu,73,False)
		if newvendor != 'quit':
			i = 0
			prefix = ''
			while not newvendor[i].isspace():
				prefix += newvendor[i]
				i+=1
			if len(prefix) > 17:
				prefix = prefix[:17]
			newaddr = prefix + newaddr[len(prefix):]
			del vendormenuwin
	elif selection == 'Load from Address Database':
		menuwin.clear()
		addrdbwin = stdscr.subwin(max_y-9,max_x-3,8,2)
		dirlist = os.listdir(addrdb)
		if len(dirlist) != 0:
			addrselection = presentMenu(addrdbwin,dirlist,40,True)
			if addrselection != 'quit':
				f = open(addrdb + '/' + addrselection)
				lines = f.readlines()
				addrdbwin.clear()
				addrselection = presentMenu(addrdbwin,lines,73,False)
				if addrselection != 'quit':
					i=0
					newaddr = ''
					while not addrselection[i].isspace():
						newaddr += addrselection[i]
						i+=1
		del addrdbwin
	elif selection == 'Addon Scripts':
		menuwin.clear()
		addonwin = stdscr.subwin(max_y-9,max_x-3,8,2)
		dirlist = os.listdir(scriptdir)
		if len(dirlist) != 0:
			scriptselection = presentMenu(addonwin,dirlist,40,True)
			stdscr.clear()
			stdscr.refresh()
			curses.nocbreak()
			stdscr.keypad(0)
			curses.echo()
			curses.endwin()
			os.system('clear')
			os.system(scriptdir + '/' + scriptselection + ' ' + dev + ' ' + addr + ' ' + newaddr)
			stdscr = curses.initscr()
			curses.noecho()
			curses.cbreak()
			stdscr.keypad(1)
			curses.curs_set(0)
			menulocation = 0
			max_y, max_x = stdscr.getmaxyx()
			menuwin = stdscr.subwin(max_y-9,max_x-3,8,2)
		del addonwin
	elif selection == 'quit':
		break

	stdscr.noutrefresh()
	curses.doupdate()

stdscr.clear()
stdscr.refresh()
curses.nocbreak()
stdscr.keypad(0)
curses.echo()
curses.endwin()
